import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/common/models/contact_form_fields.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:_private_core_network/utils.dart';
import 'package:dio/dio.dart';

import 'models/platform_user_detail.dart';
import 'models/platform_user_list.dart';

class _MyAppEndpoint {
  _MyAppEndpoint._();
  static String trackOnClick() => "v2/common/trackOnClick";
  static String getPlatformsUsersFromAdmin() =>
      "v2/common/getPlatformsUsersFromAdmin";
  static String getPlatformsUserDetailsFromAdmin() =>
      "v2/common/getPlatformsUserDetailsFromAdmin";
  static String getForms() => "v2/common/getForms";
  static String fillForm() => "v2/common/fillForm";
  static String getCountryCodeFromLatLng() =>
      "v2/common/getCountryCodeFromLatLng";
  static String getCurrencies() => "v2/common/getCurrencies";
  static String getDateFormatList() => "v2/common/getDateFormatList";
  static String getTimeFormatList() => "v2/common/getTimeFormatList";
}

abstract class MyAppApi {
  Future<NetworkResponse> trackOnClick(params);
  Future<NetworkResponse> getPlatformsUsersFromAdmin(params);
  Future<NetworkResponse> getPlatformsUserDetailsFromAdmin(params);
  Future<NetworkResponse> getForms(params);
  Future<NetworkResponse> fillForm(params);
  Future<NetworkResponse> getCountryCodeFromLatLng(params);
  Future<NetworkResponse> getCurrencies();
  Future<NetworkResponse> getDateFormatList();
  Future<NetworkResponse> getTimeFormatList();
}

class MyAppApiImp extends MyAppApi {
  @override
  Future<NetworkResponse> getDateFormatList() async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient().get(_MyAppEndpoint.getDateFormatList());
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<DateFormatModel>((i) => DateFormatModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getTimeFormatList() async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient().get(_MyAppEndpoint.getTimeFormatList());
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<TimeFormatModel>((i) => TimeFormatModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getCurrencies() async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient().get(
            _MyAppEndpoint.getCurrencies(),
            queryParameters: {'available': 1});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<CurrencyAvailable>((i) => CurrencyAvailable.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getCountryCodeFromLatLng(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient().get(
            _MyAppEndpoint.getCountryCodeFromLatLng(),
            queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) => json['countryCode'],
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getPlatformsUsersFromAdmin(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken()).get(
                _MyAppEndpoint.getPlatformsUsersFromAdmin(),
                queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) => PlatformUserList.fromJson(json),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getPlatformsUserDetailsFromAdmin(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken()).get(
                _MyAppEndpoint.getPlatformsUserDetailsFromAdmin(),
                queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) => PlatformUserDetail.fromJson(json),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> trackOnClick(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .post(_MyAppEndpoint.trackOnClick(), data: params);
        return NetworkResponse.fromResponse(
          response,
          value: response.statusCode == 200,
        );
      },
    );
  }

  @override
  Future<NetworkResponse> getForms(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient()
            .get(_MyAppEndpoint.getForms(), queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) =>
              (json as List).map((e) => ContactFormField.fromJson(e)).toList(),
        );
      },
    );
  }

  @override
  Future<NetworkResponse> fillForm(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient().post(_MyAppEndpoint.fillForm(), data: params);
        return NetworkResponse.fromResponse(
          response,
          value: response.statusCode == 200,
        );
      },
    );
  }
}
