import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:_private_core_network/utils.dart';
import 'package:dio/dio.dart';


class _MyAppEndpoint {
  _MyAppEndpoint._(); 

  static String createMediaCat() => "v2/common/createMediaCat";
  static String updateMediaCat() => "v2/common/updateMediaCat";
  static String getMediaCat() => "v2/common/getMediaCat";
  static String deleteMediaCat() => "v2/common/deleteMediaCat";

  static String uploadMedia() => "v2/common/uploadMedia";
  static String updateMedia() => "v2/common/updateMedia";
  static String getMediasByFilter() => "v2/common/getMediasByFilter";
  static String getMediaByPurpose() => "v2/common/getMediaByPurpose";
  static String deleteMedia() => "v2/common/deleteMedia"; 
}

abstract class MyAppApi { 
  Future<NetworkResponse> getMediasByFilter(params);
  Future<NetworkResponse> getMediaByPurpose(params);
  Future<NetworkResponse> getMediaCat(params);
  Future<NetworkResponse> uploadMedia(
      bytes, filePath, Map<String, dynamic> params);
  Future<NetworkResponse> createMediaCat(params);
  Future<NetworkResponse> updateMediaCat(params);
  Future<NetworkResponse> updateMedia(
      bytes, filePath, Map<String, dynamic> params);
  Future<NetworkResponse> deleteMedia(params);
  Future<NetworkResponse> deleteMediaCat(params); 
}

class MyAppApiImp extends MyAppApi { 

  @override
  Future<NetworkResponse> getMediasByFilter(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient(
                token: await appPrefs.getNormalToken())
            .get(_MyAppEndpoint.getMediasByFilter(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => ListCommonMedia.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> getMediaByPurpose(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response = await AppClient()
            .get(_MyAppEndpoint.getMediaByPurpose(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => CommonMedia.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getMediaCat(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .get(_MyAppEndpoint.getMediaCat(), queryParameters: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) =>
                (json as List).map((e) => CommonMedia.fromJson(e)).toList());
      },
    );
  }

  @override
  Future<NetworkResponse> uploadMedia(
      bytes, filePath, Map<String, dynamic> params) async {
    return await handleNetworkError(
      proccess: () async {
        String fileName = filePath.split('/').last;
        Map<String, dynamic> map = {
          "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
        };
        map.addAll(params);
        FormData formData = FormData.fromMap(map);
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .post(_MyAppEndpoint.uploadMedia(), data: formData);
        return NetworkResponse.fromResponse(response,
            converter: (json) => CommonMedia.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> createMediaCat(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .post(_MyAppEndpoint.createMediaCat(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => CommonMedia.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateMediaCat(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .post(_MyAppEndpoint.updateMediaCat(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => CommonMedia.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateMedia(
      bytes, filePath, Map<String, dynamic> params) async {
    return await handleNetworkError(
      proccess: () async {
        late Map<String, dynamic> map;
        if (bytes != null) {
          String fileName = filePath.split('/').last;
          map = {
            "fileName": MultipartFile.fromBytes(bytes, filename: fileName),
          };
        } else {
          map = {};
        }

        map.addAll(params);
        FormData formData = FormData.fromMap(map);
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .put(_MyAppEndpoint.updateMedia(), data: formData);
        return NetworkResponse.fromResponse(response,
            converter: (json) => CommonMedia.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> deleteMedia(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .delete(_MyAppEndpoint.deleteMedia(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> deleteMediaCat(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .delete(_MyAppEndpoint.deleteMediaCat(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }
}
