import 'package:_private_core/_private_core.dart';
import 'package:_private_core_network/network_resources/resources.dart';
import 'package:_private_core_network/utils.dart';
import 'package:dio/dio.dart';

import 'models/payment_billing_info.dart';

class _MyAppEndpoint {
  _MyAppEndpoint._();
  static String getBillingInfos() => "v2/payment/getBillingInfos";
  static String updateBillingInfos() => "v2/payment/updateBillingInfos";
  static String deleteBillingInfos() => "v2/payment/deleteBillingInfos";
  static String addBillingInfos() => "v2/payment/addBillingInfos";
  static String setDefaultBillingInfos() => "v2/payment/setDefaultBillingInfos";
  static String getCustomerCardsStripe() => "v2/payment/getCustomerCardsStripe";
  static String createCustomerCardStripe() => "createCustomerCardStripe";
  static String deleteCustomerCardStripe() => "deleteCustomerCardStripe";
  static String setCustomerCardDefaultStripe() =>
      "setCustomerCardDefaultStripe";
  static String getWalletInfo() => "getWalletInfo";
  static String createPurchaseByCardOnFileStripe() =>
      "createPurchaseByCardOnFileStripe";
}

abstract class MyAppApi {
  Future<NetworkResponse> getBillingInfos(params);
  Future<NetworkResponse> updateBillingInfos(params);
  Future<NetworkResponse> deleteBillingInfos(params);
  Future<NetworkResponse> addBillingInfos(params);
  Future<NetworkResponse> setDefaultBillingInfos(params);
  Future<NetworkResponse> getCustomerCardsStripe(page, pageSize);
  Future<NetworkResponse> createCustomerCardStripe(params);
  Future<NetworkResponse> deleteCustomerCardStripe(cardId);
  Future<NetworkResponse> setCustomerCardDefaultStripe(cardId);
  Future<NetworkResponse> createPurchaseByCardOnFileStripe(params);
  Future<NetworkResponse> createPurchaseByTokenStripe(params);
  Future<NetworkResponse> getWalletInfo(params);
}

class MyAppApiImp extends MyAppApi {
  @override
  Future<NetworkResponse> createPurchaseByPaymentMethodStripe(params) {
    // TODO: implement createPurchaseByPaymentMethodStripe
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> createPurchaseByTokenStripe(params) {
    // TODO: implement createPurchaseByTokenStripe
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> createPurchaseByCardOnFileStripe(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken()).post(
                _MyAppEndpoint.createPurchaseByCardOnFileStripe(),
                data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PaymentInfoModel>((i) => PaymentInfoModel.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> getWalletInfo(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .get(_MyAppEndpoint.getWalletInfo());
        return NetworkResponse.fromResponse(response,
            converter: (json) => WalletInfo.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> createCustomerCardStripe(params) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(token: token)
            .post(_MyAppEndpoint.createCustomerCardStripe(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => PaymentCardStripe.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> deleteCustomerCardStripe(cardId) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(token: token).delete(
            _MyAppEndpoint.deleteCustomerCardStripe(),
            data: {'cardId': cardId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> setCustomerCardDefaultStripe(cardId) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(token: token).put(
            _MyAppEndpoint.setCustomerCardDefaultStripe(),
            data: {'cardId': cardId});
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getCustomerCardsStripe(page, pageSize) async {
    return await handleNetworkError(
      proccess: () async {
        final token = await appPrefs.getNormalToken();
        Response response = await AppClient(token: token).get(
            _MyAppEndpoint.getCustomerCardsStripe(),
            queryParameters: {'page': page, 'page_size': pageSize});
        return NetworkResponse.fromResponse(response,
            converter: (json) => (json as List<dynamic>)
                .map<PaymentCardStripe>((i) => PaymentCardStripe.fromJson(i))
                .toList());
      },
    );
  }

  @override
  Future<NetworkResponse> addBillingInfos(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .post(_MyAppEndpoint.addBillingInfos(), data: params);
        return NetworkResponse.fromResponse(response,
            converter: (json) => PaymentBillingInfo.fromJson(json));
      },
    );
  }

  @override
  Future<NetworkResponse> updateBillingInfos(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .put(_MyAppEndpoint.updateBillingInfos(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> setDefaultBillingInfos(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .put(_MyAppEndpoint.setDefaultBillingInfos(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> deleteBillingInfos(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .put(_MyAppEndpoint.deleteBillingInfos(), data: params);
        return NetworkResponse.fromResponse(response,
            value: response.statusCode == 200);
      },
    );
  }

  @override
  Future<NetworkResponse> getBillingInfos(params) async {
    return await handleNetworkError(
      proccess: () async {
        Response response =
            await AppClient(token: await appPrefs.getNormalToken())
                .get(_MyAppEndpoint.getBillingInfos(), queryParameters: params);
        return NetworkResponse.fromResponse(
          response,
          converter: (json) => (json as List)
              .map((e) => PaymentBillingInfo.fromJson(e))
              .toList(),
        );
      },
    );
  }
}
