import 'package:_private_core_network/network_resources/resources.dart';

import 'app_api.dart';
import 'models/payment_billing_info.dart';

class PaymentRepo {
  PaymentRepo._();

  static PaymentRepo? _instance;

  factory PaymentRepo([MyAppApi? api]) {
    _instance ??= PaymentRepo._();
    _instance!._api = api ?? MyAppApiImp();
    return _instance!;
  }

  late MyAppApi _api;

  Future<bool> updateBillingInfos(params) async {
    NetworkResponse response = await _api.updateBillingInfos(params);
    return response.data ?? false;
  }

  Future<bool> deleteBillingInfos(params) async {
    NetworkResponse response = await _api.deleteBillingInfos(params);
    return response.data ?? false;
  }

  Future<bool> setDefaultBillingInfos(params) async {
    NetworkResponse response = await _api.setDefaultBillingInfos(params);
    return response.data ?? false;
  }

  Future<List<PaymentBillingInfo>> getBillingInfos([params]) async {
    NetworkResponse response = await _api.getBillingInfos(params);
    return response.data ?? [];
  }

  Future<PaymentBillingInfo?> addBillingInfos(params) async {
    NetworkResponse response = await _api.addBillingInfos(params);
    return response.data;
  }

  Future<List<PaymentCardStripe>> getCustomerCardsStripe(page, pageSize) async {
    NetworkResponse response =
        await _api.getCustomerCardsStripe(page, pageSize);
    return response.data ?? [];
  }

  Future<PaymentCardStripe?> createCustomerCardStripe(params) async {
    NetworkResponse response = await _api.createCustomerCardStripe(params);
    return response.data;
  }

  Future<NetworkResponse> createCustomerCardStripe2(params) async {
    return await _api.createCustomerCardStripe(params);
  }

  Future<bool> deleteCustomerCardStripe(cardId) async {
    NetworkResponse response = await _api.deleteCustomerCardStripe(cardId);
    return response.data ?? false;
  }

  Future<bool> setCustomerCardDefaultStripe(cardId) async {
    NetworkResponse response = await _api.setCustomerCardDefaultStripe(cardId);
    return response.data ?? false;
  }

  Future<WalletInfo?> getWalletInfo([params]) async {
    NetworkResponse response = await _api.getWalletInfo(params);
    return response.data;
  }

  Future<List<PaymentInfoModel>> createPurchaseByCardOnFileStripe(
      PaymentRequestModel request) async {
    // var params = {
    //   'purchaseType': 90,
    //   'currency': currency,
    //   'tokenType': tokenType,
    //   'priceByUnit': priceByUnit,
    //   'amount': amount,
    //   'cardId': cardId,
    // };
    NetworkResponse response =
        await _api.createPurchaseByCardOnFileStripe(request.toJson());
    return response.data ?? [];
  }
}
